
class Instance():


	def __init__(
		self,
		id = None,
		location = None,
		score = None,
		label = None,
		):

		self.id = id

		self.location = location
		self.score = score
		self.label = label


	def something():
		pass


